==> ml_pipelines_sdk-1.14.0rc0-py3-none-any.whl: tfx/dsl/experimental/conditionals/conditional_test.py <==
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tfx.dsl.experimental.conditionals.conditional."""

from typing import Any, Dict

import tensorflow as tf
from tfx.dsl.components.base import base_node
from tfx.dsl.context_managers import dsl_context_registry
from tfx.dsl.experimental.conditionals import conditional
from tfx.dsl.placeholder import placeholder


class _FakeNode(base_node.BaseNode):

  @property
  def inputs(self) -> Dict[str, Any]:
    return {}

  @property
  def outputs(self) -> Dict[str, Any]:
    return {}

  @property
  def exec_properties(self) -> Dict[str, Any]:
    return {}


class _FakePredicate(placeholder.Predicate):

  def __init__(self, name):
    super().__init__(pred_dataclass=None)
    self.name = name


class ConditionalTest(tf.test.TestCase):

  def assertPredicatesEqual(self, node, *expected_predicates):
    self.assertEqual(
        conditional.get_predicates(node, dsl_context_registry.get()),
        expected_predicates)

  def testSingleCondition(self):
    pred = _FakePredicate('pred')
    with conditional.Cond(pred):
      node1 = _FakeNode().with_id('node1')
      node2 = _FakeNode().with_id('node2')
    self.assertPredicatesEqual(node1, pred)
    self.assertPredicatesEqual(node2, pred)

  def testNestedCondition(self):
    pred1 = _FakePredicate('pred1')
    pred2 = _FakePredicate('pred2')
    with conditional.Cond(pred1):
      node1 = _FakeNode().with_id('node1')
      with conditional.Cond(pred2):
        node2 = _FakeNode().with_id('node2')
    self.assertPredicatesEqual(node1, pred1)
    self.assertPredicatesEqual(node2, pred1, pred2)

  def testReusePredicate(self):
    pred = _FakePredicate('pred')
    with conditional.Cond(pred):
      node1 = _FakeNode().with_id('node1')
    with conditional.Cond(pred):
      node2 = _FakeNode().with_id('node2')
    self.assertPredicatesEqual(node1, pred)
    self.assertPredicatesEqual(node2, pred)

  def testNestedConditionWithDuplicatePredicates(self):
    # Note: This only catches the duplication if the _same_ predicate (in terms
    # of Python object identity) is used. Ideally we would also detect
    # equivalent predicates (like __eq__) but placeholders cannot implement
    # __eq__ itself (due to its special function in creating predicates from
    # ChannelWrappedPlaceholder) and placeholders also don't offer another
    # equality function at the moment.
    pred = _FakePredicate('pred')
    with self.assertRaisesRegex(
        ValueError, 'Nested conditionals with duplicate predicates'):
      with conditional.Cond(pred):
        unused_node1 = _FakeNode().with_id('node1')
        with conditional.Cond(pred):
          unused_node2 = _FakeNode().with_id('node2')


if __name__ == '__main__':
  tf.test.main()

==> ml_pipelines_sdk-1.15.0-py3-none-any.whl: tfx/dsl/experimental/conditionals/conditional_test.py <==
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests for tfx.dsl.experimental.conditionals.conditional."""

import tensorflow as tf
from tfx.dsl.context_managers import dsl_context_registry
from tfx.dsl.context_managers import test_utils
from tfx.dsl.experimental.conditionals import conditional
from tfx.dsl.placeholder import placeholder
from tfx.orchestration import pipeline


Node = test_utils.Node


class ConditionalTest(tf.test.TestCase):

  def assertPredicatesEqual(self, node, *expected_predicates):
    self.assertEqual(
        conditional.get_predicates(node, dsl_context_registry.get()),
        expected_predicates)

  def testSingleCondition(self):
    pred = placeholder.input('foo') == 'bar'
    with conditional.Cond(pred):
      node1 = Node('node1')
      node2 = Node('node2')
    self.assertPredicatesEqual(node1, pred)
    self.assertPredicatesEqual(node2, pred)

  def testNestedCondition(self):
    pred1 = placeholder.input('foo') == 'bar'
    pred2 = placeholder.input('foo') == 'baz'
    with conditional.Cond(pred1):
      node1 = Node('node1')
      with conditional.Cond(pred2):
        node2 = Node('node2')
    self.assertPredicatesEqual(node1, pred1)
    self.assertPredicatesEqual(node2, pred1, pred2)

  def testReusePredicate(self):
    pred = placeholder.input('foo') == 'bar'
    with conditional.Cond(pred):
      node1 = Node('node1')
    with conditional.Cond(pred):
      node2 = Node('node2')
    self.assertPredicatesEqual(node1, pred)
    self.assertPredicatesEqual(node2, pred)

  def testNestedConditionWithDuplicatePredicates(self):
    # Note: This only catches the duplication if the _same_ predicate (in terms
    # of Python object identity) is used. Ideally we would also detect
    # equivalent predicates (like __eq__) but placeholders cannot implement
    # __eq__ itself (due to its special function in creating predicates from
    # ChannelWrappedPlaceholder) and placeholders also don't offer another
    # equality function at the moment.
    pred = placeholder.input('foo') == 'bar'
    with self.assertRaisesRegex(
        ValueError, 'Nested conditionals with duplicate predicates'):
      with conditional.Cond(pred):
        unused_node1 = Node('node1')
        with conditional.Cond(pred):
          unused_node2 = Node('node2')

  def testCond_Subpipeline(self):
    pred = placeholder.input('foo') == 'bar'
    with conditional.Cond(pred):
      a = Node('A')
      b = Node('B')
      p = pipeline.Pipeline(pipeline_name='p', components=[a, b])
    p_out = pipeline.Pipeline(pipeline_name='p_out', components=[p])

    # Nodes from subpipeline are isolated from the outer pipeline context thus
    # are not conditional.
    self.assertEmpty(conditional.get_predicates(a, p.dsl_context_registry))

    # Subpipeline is under conditional of the outer pipeline.
    self.assertCountEqual(
        conditional.get_predicates(p, p_out.dsl_context_registry), [pred]
    )


if __name__ == '__main__':
  tf.test.main()
